@@ -1,7 +1,7 @@
 //FJSTARTHEADER
-// $Id: fastjetfortran.cc 4442 2020-05-05 07:50:11Z soyez $
+// $Id$
 //
-// Copyright (c) 2005-2020, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
+// Copyright (c) 2005-2025, Matteo Cacciari, Gavin P. Salam and Gregory Soyez
 //
 //----------------------------------------------------------------------
 // This file is part of FastJet.
@@ -33,7 +33,22 @@
 #include "fastjet/ClusterSequenceArea.hh"
 #include "fastjet/Selector.hh"
 #include "fastjet/SISConePlugin.hh"
-//#include <thread>
+
+// Many calls below rely on caching information (input particles,
+// ClusterSequence, jets) in fastjet structures so that one can query
+// for extra information (e,g, jet constituents) after clustering.
+//
+// By default, this caching is not thread-safe.
+//
+// Uncommenting the following line results in defining all the cached
+// variables as thread_local, meaning that clustering (with caching)
+// and requesting extra information in the same thread is possible.
+//
+//#define FASTJET_FORTRAN_THREAD_LOCAL_CACHING
+
+#ifdef FASTJET_FORTRAN_THREAD_LOCAL_CACHING
+#include <thread>
+#endif
 
 using namespace std;
 using namespace fastjet;
@@ -43,71 +58,144 @@ FASTJET_BEGIN_NAMESPACE      // defined in fastjet/internal/base.hh
 /// a namespace for the fortran-wrapper which contains commonly-used
 /// structures and means to transfer fortran <-> C++
 namespace fwrapper {
+#ifdef FASTJET_FORTRAN_THREAD_LOCAL_CACHING
+  // use a thread local version of the caching so that this interface
+  // works in a (reasonably) thread-safe way.
+  thread_local vector<PseudoJet> input_particles, jets;
+  thread_local SharedPtr<JetDefinition::Plugin> plugin;
+  thread_local JetDefinition jet_def;
+  thread_local SharedPtr<ClusterSequence> cs;
+#else // FASTJET_FORTRAN_THREAD_LOCAL_CACHING not defined
   vector<PseudoJet> input_particles, jets;
   SharedPtr<JetDefinition::Plugin> plugin;
   JetDefinition jet_def;
   SharedPtr<ClusterSequence> cs;
-  // replace the above with thread local version if you want
-  // this interface to work in a (reasonably) thread-safe way.
-  // thread_local vector<PseudoJet> input_particles, jets;
-  // thread_local SharedPtr<JetDefinition::Plugin> plugin;
-  // thread_local JetDefinition jet_def;
-  // thread_local SharedPtr<ClusterSequence> cs;
+#endif // FASTJET_FORTRAN_THREAD_LOCAL_CACHING
 
-  /// helper routine to transfer fortran input particles into 
-  void transfer_input_particles(const double * p, const int & npart) {
-    input_particles.resize(0);
-    input_particles.reserve(npart);
+  /// helper routine to transform the fortran input particles into PseudoJets
+  void convert_input_particles(const double * p, const int & npart, vector<PseudoJet> &fj_particles) {
+    fj_particles.resize(0);
+    fj_particles.reserve(npart);
     for (int i=0; i<npart; i++) {
       valarray<double> mom(4); // mom[0..3]
       for (int j=0;j<=3; j++) {
          mom[j] = *(p++);
       }
       PseudoJet psjet(mom);
-      input_particles.push_back(psjet);    
+      fj_particles.push_back(psjet);    
     }
   }
 
-  /// helper routine to help transfer jets -> f77jets[4*ijet+0..3]
-  void transfer_jets(double * f77jets, int & njets) {
-    njets = jets.size();
+  /// helper routine to transfer fortran input particles into cached input_particles
+  void transfer_input_particles(const double * p, const int & npart) {
+    convert_input_particles(p, npart, input_particles);
+  }
+
+  /// helper routine to help convert fj_jets -> f77jets[4*ijet+0..3]
+  void convert_jets(const vector<PseudoJet> & fj_jets, double * f77jets, int & njets) {
+    njets = fj_jets.size();
     for (int i=0; i<njets; i++) {
       for (int j=0;j<=3; j++) {
-        *f77jets = jets[i][j];
+        *f77jets = fj_jets[i][j];
         f77jets++;
       } 
     }
   }
   
+  /// helper routine to help transfer jets -> f77jets[4*ijet+0..3]
+  void transfer_jets(double * f77jets, int & njets) {
+    convert_jets(jets, f77jets, njets);
+    // njets = jets.size();
+    // for (int i=0; i<njets; i++) {
+    //   for (int j=0;j<=3; j++) {
+    //     *f77jets = jets[i][j];
+    //     f77jets++;
+    //   } 
+    // }
+  }
+
+  /// this returns a newly-created pointer to a ClusterSequence based
+  /// on the particles and clustering parameters that have been provided
+  ClusterSequence* cluster_base(const vector<PseudoJet> & particles,
+                                const JetDefinition & jet_def,
+                                const double & ghost_maxrap = 0.0,  
+                                const int & nrepeat = 0, const double & ghost_area = 0.0) {
+    // perform the clustering
+    if ( ghost_maxrap == 0.0 ) {
+      // cluster without areas
+      return new ClusterSequence(particles,jet_def);
+    } else {
+      // cluster with areas
+      GhostedAreaSpec area_spec(ghost_maxrap,nrepeat,ghost_area);
+      AreaDefinition area_def(active_area, area_spec);
+      return new ClusterSequenceArea(particles,jet_def,area_def);
+    }
+  }
+  
   /// helper routine packaging the transfers, the clustering
   /// and the extraction of the jets
+  /// This fills the f77jets and njets but does not cache anything
+  void cluster_nocache(const double * p, const int & npart, 
+                       const JetDefinition & jet_def,
+                       double * f77jets, int & njets,
+                       const double & ghost_maxrap = 0.0,  
+                       const int & nrepeat = 0, const double & ghost_area = 0.0) {
+
+    // transfer p[4*ipart+0..3] -> particles[i]
+    vector<PseudoJet> fj_particles;
+    convert_input_particles(p, npart, fj_particles);
+
+    // cluster
+    ClusterSequence *cs_local = cluster_base(fj_particles, jet_def,
+                                             ghost_maxrap, nrepeat, ghost_area);
+    
+    // extract jets
+    vector<PseudoJet> fj_jets;
+    if (jet_def.is_spherical()){
+      fj_jets = sorted_by_E(cs_local->inclusive_jets());
+    } else {
+      fj_jets = sorted_by_pt(cs_local->inclusive_jets());
+    }
+    
+    convert_jets(fj_jets, f77jets, njets);
+
+    // release memory
+    delete cs_local;
+  }
+
+  /// helper routine packaging the transfers, the clustering
+  /// and the extraction of the jets
+  ///
+  /// This fills the f77jets and njets and caches the input particles,
+  /// the ClusterSequence and the jets for future manipulations
   void transfer_cluster_transfer(const double * p, const int & npart, 
                                  const JetDefinition & jet_def,
 				 double * f77jets, int & njets,
 				 const double & ghost_maxrap = 0.0,  
 				 const int & nrepeat = 0, const double & ghost_area = 0.0) {
 
-    // transfer p[4*ipart+0..3] -> input_particles[i]
+    // transfer p[4*ipart+0..3] -> input_particles[i]  (cached)
     transfer_input_particles(p, npart);
 
-    // perform the clustering
-    if ( ghost_maxrap == 0.0 ) {
-         // cluster without areas
-	 cs.reset(new ClusterSequence(input_particles,jet_def));
+    // cluster 
+    ClusterSequence *cs_local = cluster_base(input_particles, jet_def,
+                                             ghost_maxrap, nrepeat, ghost_area);
+
+    // cache
+    cs.reset(cs_local);
+    
+    // extract jets (into cache)
+    if (jet_def.is_spherical()){
+      jets = sorted_by_E(cs_local->inclusive_jets());
     } else {
-         // cluster with areas
-         GhostedAreaSpec area_spec(ghost_maxrap,nrepeat,ghost_area);
-         AreaDefinition area_def(active_area, area_spec);
-	 cs.reset(new ClusterSequenceArea(input_particles,jet_def,area_def));
+      jets = sorted_by_pt(cs_local->inclusive_jets());
     }
-    // extract jets (pt-ordered)
-    jets = sorted_by_pt(cs->inclusive_jets());
     
     // transfer jets -> f77jets[4*ijet+0..3]
-    transfer_jets(f77jets, njets);
- 
+    transfer_jets(f77jets, njets); 
   }
 
+  
 }
 FASTJET_END_NAMESPACE
 
@@ -116,8 +204,15 @@ using namespace fastjet::fwrapper;
 
 extern "C" {   
 
+//------------------------------------------------------------------------
+// Clustering routines
+//------------------------------------------------------------------------
+  
 /// f77 interface to SISCone (via fastjet), as defined in arXiv:0704.0292
 /// [see below for the interface to kt, Cam/Aachen & kt]
+///
+/// This caches the clustering structures for later access to extra
+/// information (constituents, ...)
 //
 // Corresponds to the following Fortran subroutine
 // interface structure:
@@ -147,17 +242,34 @@ extern "C" {
 //
 void fastjetsiscone_(const double * p, const int & npart,                   
                      const double & R, const double & f,                   
-                     double * f77jets, int & njets) {
-    
-    // prepare jet def
-    plugin.reset(new SISConePlugin(R,f));
-    jet_def = plugin.get();
-
-    // do everything
-    transfer_cluster_transfer(p,npart,jet_def,f77jets,njets);
+                     double * f77jets, int & njets) {  
+  // prepare jet def
+  plugin.reset(new SISConePlugin(R,f));
+  jet_def = plugin.get();
+  
+  // do everything
+  transfer_cluster_transfer(p,npart,jet_def,f77jets,njets);
 }
 
+/// same SISCone as above without the caching (invalidating calls to
+/// constituents, ... but making this call thread-safe)
+///
+/// Note that as of SISCone 3.0.5 SISCone itself is not guaranteed to
+/// be thread-safe
+void fastjetsisconenocache_(const double * p, const int & npart,                   
+                            const double & R, const double & f,                   
+                            double * f77jets, int & njets) {
+  // prepare jet def
+  JetDefinition::Plugin *plugin_local = new SISConePlugin(R,f);
+  JetDefinition jet_def_local = plugin_local;
 
+  // do everything
+  cluster_nocache(p,npart,jet_def_local,f77jets,njets);
+  
+  // release memory
+  delete plugin_local;
+}
+ 
 
 /// f77 interface to SISCone (via fastjet), as defined in arXiv:0704.0292
 /// [see below for the interface to kt, Cam/Aachen & kt]
@@ -197,15 +309,33 @@ void fastjetsisconewitharea_(const double * p, const int & npart,
                      const double & R, const double & f,                   
                      const double & ghost_rapmax, const int & nrepeat, const double & ghost_area,
                      double * f77jets, int & njets) {
-    
-    // prepare jet def
-    plugin.reset(new SISConePlugin(R,f));
-    jet_def = plugin.get();
-
-    // do everything
-    transfer_cluster_transfer(p,npart,jet_def,f77jets,njets,ghost_rapmax,nrepeat,ghost_area);
+  // prepare jet def
+  plugin.reset(new SISConePlugin(R,f));
+  jet_def = plugin.get();
+  
+  // do everything
+  transfer_cluster_transfer(p,npart,jet_def,f77jets,njets,ghost_rapmax,nrepeat,ghost_area);
 }
 
+/// same SISCone+area as above without the caching (invalidating calls
+/// to constituents, ... but making this call thread-safe)
+///
+/// Note that as of SISCone 3.0.5 SISCone itself is not guaranteed to
+/// be thread-safe
+void fastjetsisconewithareanocache_(const double * p, const int & npart,                   
+                                    const double & R, const double & f,                   
+                                    const double & ghost_rapmax, const int & nrepeat, const double & ghost_area,
+                                    double * f77jets, int & njets) {
+  // prepare jet def
+  JetDefinition::Plugin *plugin_local = new SISConePlugin(R,f);
+  JetDefinition jet_def_local = plugin_local;
+
+  // do everything
+  cluster_nocache(p,npart,jet_def_local,f77jets,njets,ghost_rapmax,nrepeat,ghost_area);
+
+  // release memory
+  delete plugin_local;
+}
 
 
 /// f77 interface to the pp generalised-kt (sequential recombination)
@@ -248,20 +378,44 @@ void fastjetppgenkt_(const double * p, const int & npart,
                      const double & R, const double & palg,
                      double * f77jets, int & njets) {
     
-    // prepare jet def
-    if (palg == 1.0) {
-      jet_def = JetDefinition(kt_algorithm, R);
-    }  else if (palg == 0.0) {
-      jet_def = JetDefinition(cambridge_algorithm, R);
-    }  else if (palg == -1.0) {
-      jet_def = JetDefinition(antikt_algorithm, R);
-    } else {
-      jet_def = JetDefinition(genkt_algorithm, R, palg);
-    }
-
-    // do everything
-    transfer_cluster_transfer(p,npart,jet_def,f77jets,njets);
+  // prepare jet def
+  if (palg == 1.0) {
+    jet_def = JetDefinition(kt_algorithm, R);
+  }  else if (palg == 0.0) {
+    jet_def = JetDefinition(cambridge_algorithm, R);
+  }  else if (palg == -1.0) {
+    jet_def = JetDefinition(antikt_algorithm, R);
+  } else {
+    jet_def = JetDefinition(genkt_algorithm, R, palg);
+  }
+  
+  // do everything
+  transfer_cluster_transfer(p,npart,jet_def,f77jets,njets);
 }
+  
+/// same pp generalised-kt as above without the caching (invalidating
+/// calls to constituents, ... but making this call thread-safe)
+void fastjetppgenktnocache_(const double * p, const int & npart,                   
+                            const double & R, const double & palg,
+                            double * f77jets, int & njets) {
+  
+  // prepare jet def
+  JetDefinition jet_def_local;
+  if (palg == 1.0) {
+    jet_def_local = JetDefinition(kt_algorithm, R);
+  }  else if (palg == 0.0) {
+    jet_def_local = JetDefinition(cambridge_algorithm, R);
+  }  else if (palg == -1.0) {
+    jet_def_local = JetDefinition(antikt_algorithm, R);
+  } else {
+    jet_def_local = JetDefinition(genkt_algorithm, R, palg);
+  }
+  
+  // do everything
+  cluster_nocache(p,npart,jet_def_local,f77jets,njets);
+}
+
+
 
 /// a routine that provides similar f77 functionality to fastjetppgenkt_, 
 /// but for the e+e- algorithms instead of the pp ones; note this 
@@ -278,8 +432,20 @@ void fastjeteegenkt_(const double * p, const int & npart,
   transfer_cluster_transfer(p,npart,jet_def,f77jets,njets);
 }
 
+/// same ee generalised-kt as above without the caching (invalidating
+/// calls to constituents, ... but making this call thread-safe)
+void fastjeteegenktnocache_(const double * p, const int & npart,                   
+                            const double & R, const double & palg,
+                            double * f77jets, int & njets) {
+    
+  // prepare jet def
+  JetDefinition jet_def_local = JetDefinition(ee_genkt_algorithm, R, palg);
+  
+  // do everything
+  cluster_nocache(p,npart,jet_def_local,f77jets,njets);
+}
 
-
+  
 /// f77 interface to the pp generalised-kt (sequential recombination)
 /// algorithms, as defined in arXiv.org:0802.1189, which includes
 /// kt, Cambridge/Aachen and anti-kt as special cases.
@@ -342,6 +508,40 @@ void fastjetppgenktwitharea_(const double * p, const int & npart,
 }
 
 
+/// same pp generalised-kt+area as above without the caching
+/// (invalidating calls to constituents, ... but making this call
+/// thread-safe)
+void fastjetppgenktwithareanocache_(const double * p, const int & npart,                   
+                                    const double & R, const double & palg,
+                                    const double & ghost_rapmax, const int & nrepeat, const double & ghost_area,
+                                    double * f77jets, int & njets) {
+    
+  // prepare jet def
+  JetDefinition jet_def_local;
+  if (palg == 1.0) {
+    jet_def_local = JetDefinition(kt_algorithm, R);
+  }  else if (palg == 0.0) {
+    jet_def_local = JetDefinition(cambridge_algorithm, R);
+  }  else if (palg == -1.0) {
+    jet_def_local = JetDefinition(antikt_algorithm, R);
+  } else {
+    jet_def_local = JetDefinition(genkt_algorithm, R, palg);
+  }
+  
+  // do everything
+  cluster_nocache(p,npart,jet_def_local,f77jets,njets,ghost_rapmax,nrepeat,ghost_area);
+}
+
+
+//------------------------------------------------------------------------
+// routines which access the cached particles/CluterSequence/jets
+//
+// Note that this is not available in the "nocache" versions of the
+// above clustering routines These routines may not work properly in a
+// threaded environment unless the
+// FASTJET_FORTRAN_THREAD_LOCAL_CACHING flag is defined
+//------------------------------------------------------------------------
+  
 /// f77 interface to provide access to the constituents of a jet found
 /// in the jet clustering with one of the above routines.
 ///
